﻿/*
 * (C) 2008 Peter O.
 * Permission is granted to anyone to copy, modify,
 * and redistribute this file.  Attribution to the
 * author would be appreciated but is not required.
 * No warranties are provided and all liabilities
 * are disclaimed.
 */

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using PeterO;

namespace FileShredder
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		
		private string[] data=null;
		
		public MainForm()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
		}
		
		void MainFormDragDrop(object sender, DragEventArgs e)
		{
			string[] data=e.Data.GetData(DataFormats.FileDrop) as string[];
			this.data=data;
			// Can't handle drag-drop event here, since it will cause
			// the source window to hang during the deletion
		}
		
		void MainFormDragEnter(object sender, DragEventArgs e)
		{
			string[] data=e.Data.GetData(DataFormats.FileDrop) as string[];
			if(data!=null){
				e.Effect= DragDropEffects.Copy;
			}
		}
		
		void Timer1Tick(object sender, EventArgs e)
		{
			if(this.data!=null){
				string[] thisData=this.data;
				this.data=null;
				System.Text.StringBuilder builder=new System.Text.StringBuilder();
				int filecount=0;
				if(thisData.Length<10){
					builder.AppendLine("Are you sure you want to delete the following?");
					builder.AppendLine("");
					foreach(string file in thisData){
						if(File.Exists(file)||Directory.Exists(file)){
							builder.AppendLine(file);
							filecount++;
						}
					}
				} else {
					builder.AppendFormat("Are you sure you want to delete these {0} files?",
					                     thisData.Length);
					foreach(string file in thisData){
						if(File.Exists(file)||Directory.Exists(file)){
							filecount++;
						}
					}
				}
				if(filecount>0 &&
				   MessageBox.Show(builder.ToString(),
				                   "File Shredder",
				                   MessageBoxButtons.YesNo)==DialogResult.Yes){
					Form form=new ProgressForm(thisData);
					form.ShowDialog(this);
				}
			}
		}
	}
}
