/*
 * (C) 2008 Peter O.
 * Permission is granted to anyone to copy, modify,
 * and redistribute this file.  Attribution to the
 * author would be appreciated but is not required.
 * No warranties are provided and all liabilities
 * are disclaimed.
 */

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using PeterO;
using System.Text;
using System.IO;
namespace FileShredder
{
	/// <summary>
	/// Description of ProgressForm.
	/// </summary>
	public partial class ProgressForm : Form
	{
		private string[] files;
		private volatile bool done;
		private volatile int progress;
		private string currentFile=String.Empty;
		private object currentFileLock=new Object();
		
		public string CurrentFile {
			get {
				lock(currentFileLock){
					return currentFile;
				}
			}
			set {
				lock(currentFileLock){
					currentFile=value;
				}
			}
		}
		// No need to make thread safe, since
		// used by only one thread, the shredder thread
		private Random random=new Random();
		public ProgressForm(string[] files)
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			this.files=files;
			
		}
		
		public string GetNewRandomName(string fileName){
			string randomChars="ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
			StringBuilder builder=new StringBuilder();
			// Create a random 8.3 filename
			for(int i=0;i<8;i++){
				builder.Append(randomChars[random.Next(randomChars.Length)]);
			}
			builder.Append('.');
			for(int i=0;i<3;i++){
				builder.Append(randomChars[random.Next(randomChars.Length)]);
			}
			string absFileName=Path.GetFullPath(fileName);
			string dirPath=Path.GetDirectoryName(absFileName);
			if(dirPath==null){
				return fileName; // It's a root directory
			} else {
				// Append new filename to root
				return Path.Combine(dirPath,builder.ToString());
			}
			
		}
		
		public string RenameToRandomName(string fileName){
			string newPath=GetNewRandomName(fileName);
			try {
				// Move file to a random name before deleting
				if(File.Exists(fileName))
					File.Move(fileName,newPath);
				else if(Directory.Exists(fileName))
					Directory.Move(fileName,newPath);
				return newPath;
			} catch(IOException){
				// Ignore, it should be rare for
				// the new name to exist
				return fileName;
			} catch(UnauthorizedAccessException){
				// Ignore, moving the file is not
				// essential to the shredding process
				return fileName;
			}
		}
		
		private void StartFileShred(string file){
			this.CurrentFile=file;
			if(File.Exists(file)){
				string movedFile=RenameToRandomName(file);
				Shredder.Shred(movedFile);
			} else if(Directory.Exists(file)){
				string[] allFiles;
				try {
					allFiles=Directory.GetFiles(file);
				} catch(IOException){
					return;
				}
				foreach(string f in allFiles){
					StartFileShred(f);
				}
				try {
					allFiles=Directory.GetDirectories(file);
				} catch(IOException){
					return;
				}
				foreach(string f in allFiles){
					StartFileShred(f);
				}
				// Move directory to a new name
				// before deleting
				file=RenameToRandomName(file);
				try {
					Directory.Delete(file);
				} catch(IOException){
					return;
				}
			}
		}
		
		private void ShredderThread(){
			for(int i=0;i<files.Length;i++){
				StartFileShred(files[i]);
				this.progress=(i+1)*10000/files.Length;
				if(this.done)
					break;
			}
			this.done=true;
		}
		
		void ProgressFormLoad(object sender, EventArgs e)
		{
			this.Location=new Point(this.Owner.Location.X+20,
			                        this.Owner.Location.Y+20);
			new Thread(ShredderThread).Start();
			this.timer1.Start();
		}
		
		
		void Timer1Tick(object sender, EventArgs e)
		{
			this.fileBeingDeleted.Text=Path.GetFileName(this.CurrentFile);
			this.progressBar.Value=this.progress;
			if(this.done)
				this.Close();
		}
		
		void Button1Click(object sender, EventArgs e)
		{
			// Cancels operation
			this.done=true;
		}
	}
}
